@@ -39,8 +39,4 @@
 #define BOOST_HAS_STDINT_H
 
 // boilerplate code:
-#include <boost/config/posix_features.hpp>
-
-
-
-
+#include <boost/config/detail/posix_features.hpp>
